module Rubapi
  class Controller
    attr_accessor :request, :response, :params

    def initialize(response)
      self.response = response
    end

    def index
      not_defined
    end

    def show
      not_defined
    end

    def create
      not_defined
    end

    def update
      not_defined
    end

    def delete
      not_defined
    end

    def not_defined
      self.response.tap do |r|
        r.status = 417
        r.content = "{\"errors\":[\"not defined\"]}"
        r.send_response
      end
    end

    def send_response(content)
      self.response.tap do |r|
        r.status = 200

        begin
          r.content = JSON.generate(attach_data(content))
        rescue
          r.content = "{\"errors\":[\"can't generate a json object\"]}"
          r.status = 417 #Execution failed
        ensure
          r.send_response
        end
      end
    end

    def run(route)
      if route.nil?
        response.status = 404
        response.send_response

        return response
      end

      route[:controller] ||= self.class
      controller = route[:controller].new(self.response)
      controller.request = self.request
      controller.request.pagination = route[:pagination] unless route[:pagination].nil?
      controller.send(route[:method])
    end

    def process_defer(reql_expression)
      get_conn = proc{ ReconnectionPool.get }
      callback = proc{|conn|
        reql_expression.run(conn) do |row|
          yield(row.to_a) if block_given?
        end

        ReconnectionPool.return conn
        send_response(nil) unless block_given?
      }

      EM.defer get_conn, callback
    end

    # If you try to respond with a reql.expr, it ensures that will return with a Hash(Dict) format
    def format_response(response)
      response.map {|re|
        { :"#{re.first}" => re.last }
      }.inject({}) {|sum, re| sum.merge!(re)}
    end

    def attach_data(data)
      @content ||= { errors: [] }
      @content[:data] = data;

      @content
    end

  end
end
